// ../../../warn-or-throw-incompatible.ts
var warnOrThrowIncompatible = (id, args = [], extraInfo) => {
  if (typeof CF_WORKER_COMPAT_ON_INCOMPATIBLE !== "undefined" && CF_WORKER_COMPAT_ON_INCOMPATIBLE !== "ignore") {
    let printedArgs = [];
    if (typeof args[0] === "string" && args[0].length < 100) {
      printedArgs.push(`"${args[0]}"`);
    } else {
      printedArgs.push("...");
    }
    if (args.length > 1) {
      printedArgs.push("...");
    }
    const printedArgsStr = printedArgs.join(", ");
    const warningOrError = `Called \`${id}(${printedArgsStr})\`, ${extraInfo ? `${extraInfo}, ` : ""}but this is not available in Workers`;
    if (CF_WORKER_COMPAT_ON_INCOMPATIBLE === "warn") {
      console.warn(warningOrError);
    } else {
      throw new Error(warningOrError);
    }
  }
};

// ../../node-globals/process/inject.ts
var workerGlobal_process = {
  title: "cloudflare-worker",
  browser: true,
  env: {},
  argv: [],
  version: "",
  versions: {},
  on: (...args) => warnOrThrowIncompatible("process.on", args),
  addListener: (...args) => warnOrThrowIncompatible("process.addListener", args),
  once: (...args) => warnOrThrowIncompatible("process.once", args),
  off: (...args) => warnOrThrowIncompatible("process.off", args),
  removeListener: (...args) => warnOrThrowIncompatible("process.removeListener", args),
  removeAllListeners: (...args) => warnOrThrowIncompatible("process.removeAllListeners", args),
  emit: (...args) => warnOrThrowIncompatible("process.emit", args),
  prependListener: (...args) => warnOrThrowIncompatible("process.prependListener", args),
  prependOnceListener: (...args) => warnOrThrowIncompatible("process.prependOnceListener", args),
  eventNames: () => {
    warnOrThrowIncompatible("process.prependOnceListener");
    return [];
  },
  listeners: (...args) => {
    warnOrThrowIncompatible("process.listeners", args);
    return [];
  },
  binding: (...args) => warnOrThrowIncompatible("process.binding", args),
  cwd: () => {
    warnOrThrowIncompatible("process.cwd");
    return "/";
  },
  chdir: (...args) => warnOrThrowIncompatible("process.chdir", args),
  umask: () => {
    warnOrThrowIncompatible("process.umask");
    return 0;
  },
  nextTick: () => warnOrThrowIncompatible("process.nextTick")
};

// process.ts
var process_default = workerGlobal_process;
export {
  process_default as default
};
